/*=============================================================================
author        :Walter Schreppers & Nhan Truong
filename      :Connection.java
created       :20/12/2000
modified      :26/01/2001
version       :1
bugreport(log):/
=============================================================================*/
import java.net.*;
import java.io.*;

/* This class handles a single connection for a incoming request.
 */

public class Connection extends Thread {

	private Socket client;

	public Connection(Socket clientsock){
		client=clientsock;
		start();
	}
	public void run(){
		try{
			Output.out.println("Serving incoming request ...");

	        /* parse incoming request string and create
	         * corresponding 'Request' and 'Response' objects
	         */
			Request req = new Request(client.getInputStream());
			Response res = new Response(client.getOutputStream());
			
			/* Get client's IP address */
			String host = client.getInetAddress().getHostAddress();
			Output.out.println("--> (" + host + "): " + req.getRequestLine());
			
			/* after parsing the incoming request, this request is serviced here.
			 * first we'll check in the request line if an object is
			 * specified. (usually a filename -> .html, .jsp, ...).
			 * If no object is specified, than a default response is generated.
			 * Else we check the extension of the specified filename.
			 * 2 cases are handled: .jsp files and other files with static content.
			 */
			 
			if(req.getObject().length()==0){
				outputDefault(res.getOutputStream());	// send default output
				Output.out.println("<-- (" + host + "): dynamic content");
			}
			else{
				String extension = req.getObjectInfo();
				if( (extension != null) && (extension.equals("jsp")) ) {	// jsp request
					Output.out.println("JSP request ... " + req.getObject());
					JSPEngine engine = new JSPEngine();
					engine.handle(req, res);			// handle jsp request
					Output.out.println("<-- (" + host + "): dynamic content (" + req.getPath() + req.getObject() + ")");
				}
				else {		// other request (file request)
					FileSender send = new FileSender(req, res);	// handle file request
					send.handle();
					Output.out.println("<-- (" + host + "): " + send.getFullName());
				}
			}
			
			/* Request processing is now done. Connection is closed here */
			client.close();
			Output.out.println("Request processing done ... (" + host + ") Connection closed.\n");
		}
		catch (IOException io){
			Output.err.println(io.getMessage());
			io.printStackTrace();
		}
		catch (Exception e) {
			e.printStackTrace();
			Output.err.println(e.getMessage());
			Output.err.println(e.getLocalizedMessage());
		}
	}
	
	private void outputDefault(PrintStream out){
			out.println("HTTP/1.0 200\r\n"
			+ "Content-Type: text/html\r\n\r\n"
			+ "<HTML> <TITLE>WN HTTPServer</TITLE><BODY>\r\n"
			+ "<CENTER><H1>WN HTTPServer</H1><BR>\r\n"
			+ "<B>Netwerk Toepassingen<BR>\r\n"
			+ "Academiejaar 2000-2001<BR>\r\n"
			+ "Authors: Walter Schreppers & Nhan Truong</CENTER></B><HR> <BR><BR>\r\n"
 			+ "Press refresh a couple of times to see this random number change: \r\n"
			+ Math.random()
			+ "<BR>The page you are looking for should be <a href=index.html>here</a>\r\n");
	}
}


