/*=============================================================================
author        :Walter Schreppers
filename      :JspEngine.java
created       :20/12/2000
modified      :27/01/2001
version       :3
bugreport(log):/
=============================================================================*/
import java.io.*;
import java.net.*;
import java.util.*;
import java.math.*;

/* This class handles JSP requests.  It creates a corresponding 'JspPage' object,
 * sends signals to parse and compile it (if necessary) and finally makes it
 * execute, to return the dynamic content of the page.
 */
class JSPEngine{

	public String classPath;
	public String classDir;
	
	public JSPEngine(){
		classPath = new String("classes/");
		classDir = new String(JspPage.root);
	}
	public void handle(Request req, Response res){
		try{
			/* create the corresponding 'JspPage' object */
			JspPage page = new JspPage(req.getPath(), req.getObjectName());

			/* first check if the page needs to be compiled */
			if (!page.modified()) {		// no compilation needed
				page.execute(req, res);	// execute it
			} else {
				page.jspToJava();		// parse JSP page (jsp to java)
				Output.out.print(page.getFullName() + ".java:  compiling ... ");
				
				/* compile the java vesion of the JSP page */
				Process compilation = Runtime.getRuntime().exec("javac"
					+ " -d " + classDir + req.getPath()
					+ " -classpath " + classPath + " " 
					+ JspPage.root + req.getPath() + page.getName()+ ".java"); //compile source			
				String errline;

				/* check for compilation errors.
				 * If compilation errors occured, the error stream is output.
				 * Else we conclude the page is compiled successfully, and execute it.
				 */
				BufferedReader errStream = new BufferedReader(
									new InputStreamReader(compilation.getErrorStream()));
				if((errline=errStream.readLine())!=null) {
					Output.out.println("compile error!");
					res.setContentType("text/html");
					res.sendHeaders();
					printErrors(page,res.getOutputStream(),errline,errStream);
				} else {
					Output.out.println("done successfully!");
					page.execute(req, res);
				}
			}
		}
		catch (FileNotFoundException fnf) {
			fnf.printStackTrace();
			Output.err.println(fnf.getLocalizedMessage());
			String msg = new String("<BIG>Error " + res.SC_NOT_FOUND + ":</BIG><BR>");
			msg += "<BIG>Requested object not found on server</BIG>";
			res.sendError(res.SC_NOT_FOUND, msg);
		}
		catch (Exception e) {
			System.out.println("exception e");
			e.printStackTrace();
			Output.err.println(e.getLocalizedMessage());
			String msg = new String("<BIG>status 505</BIG><BR>\r\n");
			msg += "<BIG>Internal Server Error:</BIG><BR>\r\n";
			msg += e.getMessage();
			res.sendError(res.SC_INTERNAL_SERVER_ERROR, msg);
		}
	}

	private void printErrors(JspPage page,PrintStream out, String errLine,BufferedReader errStream){
		String errors = new String();
		try{
			errors="<center><h1>Compile errors in '"+page.getName()+".jsp'</h1></center><br>\n";
			errors+=errLine+"<br>\n";
			while((errLine = errStream.readLine())!=null) errors += errLine + "<br>\n";
			out.println(errors);
		}
		catch(IOException io){
			io.printStackTrace();
		}
	}	
}
