/*=============================================================================
author        :Walter Schreppers & Nhan Truong
filename      :Request.java
created       :20/12/2000
modified      :27/01/2001
version       :2
bugreport(log):/
=============================================================================*/
import java.io.*;
import java.util.*;

public class Request {
	
	public static final int UNKNOWN = 0;
	public static final int GET = 1;
	public static final int POST = 2;
	
	public static final String ROOT_DIR = "root/";
	
	private int reqMethod = UNKNOWN;
	private String reqStr = null;
	private String uri = null;
	private String path = null;
	private String queryString = null;
	private String obj = null;
	private Hashtable reqParams = new Hashtable();
	private BufferedReader instream;
	
	public Request(InputStream instream) {
		try {
			this.instream = new BufferedReader(new InputStreamReader(instream));
			this.reqStr = this.instream.readLine();
			if (reqStr == null)
				reqStr = new String("");
		} catch (IOException e) {
			e.printStackTrace();
		}
			
		reqMethod = parseMethod(reqStr);
		path = parsePath(reqStr);
		queryString = parseQueryString(reqStr);
		parseParameters(reqStr);
	}
	
	protected int parseMethod(String reqline) {
		if (reqline.startsWith("GET")) {
			return GET;
		}
		if (reqline.startsWith("POST")) {
			return POST;
		}
		return UNKNOWN;
	}
	
	protected String parsePath(String path) {
		int index = -1;
		if ((index = path.indexOf('/')) < 0)
			return null;
		path = path.substring(index + 1);

		if ((index = path.indexOf('?')) > -1)
			path = path.substring(0, index);
		else if ((index = path.indexOf(' ')) > -1)
			path = path.substring(0, index);

		if ((index = path.lastIndexOf('/')) > -1) {
			obj = path.substring(index + 1);
			path = path.substring(0, index + 1);
		} else {
			obj = path;
			path = "";
		}
			
		return path;
	}
	
	protected String parseQueryString(String queryString) {
		if (queryString == null)
			return null;
			
		int begin, end = -1;;
		if ((begin = queryString.indexOf('?')) < 0)
			return null;
		queryString = queryString.substring(begin + 1);
		if ((end = queryString.indexOf(' ')) > -1)
			queryString = queryString.substring(0, end);
		
		return queryString;
	}
	
	protected void parseParameters(String reqline) {
		int i = reqline.indexOf('?');
		if (i < 0) return;

		String parameters = reqline.substring(i + 1);				/* strip off useless prefix */
		int iend = parameters.indexOf(' ');
		if (i > -1) parameters = parameters.substring(0, iend);		/* strip off useless suffix */
		
		String parname = null;
		String parvalue = null;
		for (;i > -1; parameters = parameters.substring(i + 1)) {
			i = parameters.indexOf('=');
			if (i < 0) break;
			parname = parameters.substring(0,i);
			parameters = parameters.substring(i+1);
			i = parameters.indexOf('&');
			if (i < 0) parvalue = parameters;
			else parvalue = parameters.substring(0,i);
			this.reqParams.put(parname, parvalue);
		}
	}
	
	public String getRequestLine() {
		return reqStr;
	}
	
	public String getQueryString() {
		return queryString;
	}
	
	public String getPath() {
		return path;
	}
	
	public String getMethod() {
		switch (reqMethod) {
			case GET:
				return "GET";
			case POST:
				return "POST";
			default: return "UNKNOWN";
		}
	}
	
	public String getObject(){
		//String s = reqStr.substring(reqStr.indexOf('/')+1,reqStr.lastIndexOf(' '));
		return obj;
	}
	
	public String getObjectName() {
		int index = obj.indexOf('.');
		if (index > -1)
			return obj.substring(0, index);
		return obj;
	}
	
	public String getObjectInfo() {
		String info = null;
		int index = obj.indexOf('.');
		if (index > -1)
			info = obj.substring(index + 1);
		return info;
	}
		
	public String dir(){
		return "root/";
	}
	
	public String getParameter(String name) {
		return (String) reqParams.get(name);
	}
	
	public BufferedReader getInputStream() {
		return instream;
	}
	
}


