/*=============================================================================
author        :Walter Schreppers & Nhan Truong
filename      :Response.java
created       :29/01/2000
modified      :29/01/2000
version       :2
bugreport(log):/
=============================================================================*/
import java.io.*;

public class Response {

	public static final int SC_ACCEPTED = 202;
	public static final int SC_BAD_GATEWAY = 502;
	public static final int SC_BAD_REQUEST = 400;
	public static final int SC_CONFLICT= 409;
	public static final int SC_CONTINUE= 100;
	public static final int SC_CREATED= 201;
	public static final int SC_FORBIDDEN = 403;
	public static final int SC_GATEWAY_TIMEOUT = 504;
	public static final int SC_GONE = 410;
	public static final int SC_HTTP_VERSION_NOT_SUPPORTED = 505;
	public static final int SC_INTERNAL_SERVER_ERROR = 500;
	public static final int SC_LENGTH_REQUIRED = 411;
	public static final int SC_METHOD_NOT_ALLOWED = 405;
	public static final int SC_MOVED_PERMANENTLY = 301;
	public static final int SC_MOVED_TEMPORARILY= 302;
	public static final int SC_MULTIPLE_CHOICES = 300;
	public static final int SC_NO_CONTENT = 204;
	public static final int SC_NON_AUTHORITATIVE_INFORMATION = 203;
	public static final int SC_NOT_ACCEPTABLE = 406;
	public static final int SC_NOT_FOUND = 404;
	public static final int SC_NOT_IMPLEMENTED = 501;
	public static final int SC_NOT_MODIFIED = 304;
	public static final int SC_OK = 200;
	public static final int SC_PARTIAL_CONTENT = 206;
	public static final int SC_PAYMENT_REQUIRED = 402;
	public static final int SC_PRECONDITION_FAILED = 412;
	public static final int SC_PROXY_AUTHENTICATION_REQUIRED = 407;
	public static final int SC_REQUEST_ENTITY_TOO_LARGE = 413;
	public static final int SC_REQUEST_TIMEOUT = 408;
	public static final int SC_REQUEST_URI_TOO_LONG = 414;
	public static final int SC_RESET_CONTENT = 205;
	public static final int SC_SEE_OTHER = 303;
	public static final int SC_SERVICE_UNAVAILABLE = 503;
	public static final int SC_SWITCHING_PROTOCOLS = 101;
	public static final int SC_UNAUTHORIZED = 401;
	public static final int SC_UNSUPPORTED_MEDIA_TYPE = 415;
	public static final int SC_USE_PROXY = 305;
	
	public static final String PROTOCOL = "HTTP/1.0";
	
	private PrintStream outstream;
	
	private String contentType = null;
	private int statusCode = Response.SC_OK;
	private String message = getStatusMessage(Response.SC_OK);
	private int contentLength = -1;
	
	public Response(OutputStream outstream) {
		this.outstream = new PrintStream(outstream);
	}
	
    protected String getStatusMessage(int status) {
		switch (status) {
			case SC_OK: return ("OK");
			case SC_ACCEPTED: return ("Accepted");
			case SC_BAD_GATEWAY: return ("Bad Gateway");
			case SC_BAD_REQUEST: return ("Bad Request");
			case SC_CONFLICT: return ("Conflict");
			case SC_CONTINUE: return ("Continue");
			case SC_CREATED: return ("Created");
			case SC_FORBIDDEN: return ("Forbidden");
			case SC_GATEWAY_TIMEOUT: return ("Gateway Timeout");
			case SC_GONE: return ("Gone");
			case SC_HTTP_VERSION_NOT_SUPPORTED: return ("HTTP Version Not Supported");
			case SC_INTERNAL_SERVER_ERROR: return ("Internal Server Error");
			case SC_LENGTH_REQUIRED: return ("Length Required");
			case SC_METHOD_NOT_ALLOWED: return ("Method Not Allowed");
			case SC_MOVED_PERMANENTLY: return ("Moved Permanently");
			case SC_MOVED_TEMPORARILY: return ("Moved Temporarily");
			case SC_MULTIPLE_CHOICES: return ("Multiple Choices");
			case SC_NO_CONTENT: return ("No Content");
			case SC_NON_AUTHORITATIVE_INFORMATION: return ("Non-Authoritative Information");
			case SC_NOT_ACCEPTABLE: return ("Not Acceptable");
			case SC_NOT_FOUND: return ("Not Found");
			case SC_NOT_IMPLEMENTED: return ("Not Implemented");
			case SC_NOT_MODIFIED: return ("Not Modified");
			case SC_PARTIAL_CONTENT: return ("Partial Content");
			case SC_PAYMENT_REQUIRED: return ("Payment Required");
			case SC_PRECONDITION_FAILED: return ("Precondition Failed");
			case SC_PROXY_AUTHENTICATION_REQUIRED: return ("Proxy Authentication Required");
			case SC_REQUEST_ENTITY_TOO_LARGE: return ("Request Entity Too Large");
			case SC_REQUEST_TIMEOUT: return ("Request Timeout");
			case SC_REQUEST_URI_TOO_LONG: return ("Request URI Too Long");
			case SC_RESET_CONTENT: return ("Reset Content");
			case SC_SEE_OTHER: return ("See Other");
			case SC_SERVICE_UNAVAILABLE: return ("Service Unavailable");
			case SC_SWITCHING_PROTOCOLS: return ("Switching Protocols");
			case SC_UNAUTHORIZED: return ("Unauthorized");
			case SC_UNSUPPORTED_MEDIA_TYPE: return ("Unsupported Media Type");
			case SC_USE_PROXY: return ("Use Proxy");
			default: return ("HTTP Response Status " + status);
		}
	}

	public PrintStream getOutputStream() {
		return outstream;
	}
	
	public int getStatusCode() {
		return this.statusCode;
	}
	
	public String getMessage() {
		return message;
	}
	
	public String getContentType() {
		return this.contentType;
	}
	
	public int getContentLength() {
		return this.contentLength;
	}
	
	public void setStatusCode(int sc) {
		this.statusCode = sc;
	}
	
	public void setMessage(String msg) {
		message = msg;
	}
	
	public void setContentType(String ct) {
		this.contentType = ct;
	}
	
	public void setContentLength(int cl) {
		this.contentLength = cl;
	}
	
	protected void sendHeaders() {
		outstream.print(PROTOCOL);
		outstream.print(" ");
		outstream.print(statusCode);
		if (message != null) {
			outstream.print(" ");
			outstream.print(message);
		}
		outstream.print("\r\n");
		if (getContentType() != null) {
			outstream.print("Content-Type: " + getContentType() + "\r\n");
		}
		if (getContentLength() >= 0) {
			outstream.print("Content-Length: " + getContentLength() + "\r\n");
		}
		outstream.print("\r\n");
	}
	
	public void sendError(int status, String msg) {
		setStatusCode(status);
		setMessage(getStatusMessage(status));
		sendHeaders();
		outstream.print(msg + "\r\n");
	}
				
}