import java.io.*;
import javax.swing.*;
import javax.swing.event.*;
import java.awt.*;
import java.awt.event.*;
import java.beans.*;

/* This class represents a window, in whick the server can run.
 * Following components are provided:
 *	- 'start' button:		to start the server
 *	- 'stop' buttont:		to stop the server
 *	- 'messages' text area:	for regular server messages
 *	- 'error' text area:	to output the error messages
 *	- some labels to display the server status
 *	- 'port' textfield:		with which the user can specify the port the server should listen to
 */

class ServerWindow
extends JFrame
implements ActionListener, ComponentListener {	
	
	private static final Color LABEL_COLOR = new Color(30, 60, 85);
	private static final Color ONLINE_COLOR = new Color(0,120,0);
	private static final Color OFFLINE_COLOR = new Color(120, 0, 0);
	
	private HTTPServer server = null;;
	
	private JButton start = new JButton("Start");
	private JButton stop = new JButton("Stop");
	private JTextArea messages = new JTextArea();
	private JTextArea errors = new JTextArea();

	private JLabel port = new JLabel("Server listening on port", SwingConstants.LEFT);
	private JTextField port_value = new JTextField("",4);
	private JLabel status = new JLabel("Server status: ", SwingConstants.LEFT);
	private JLabel status_value = new JLabel("Offline", SwingConstants.LEFT);
	
	private JScrollBar msgScrollBar;
	private JScrollBar errsScrollBar;
	
	public ServerWindow(int port) {
		setSize(new Dimension(550, 400));
		//setPreferredSize(new Dimension(550,400));
		setTitle("WN HTTP Server - Walter Schreppers & Nhan Truong");
		port_value.setText(new Integer(port).toString());
		init();
	}
	
	
	
	public void init() {
		Container content = getContentPane();
		content.setLayout(new BorderLayout());

		start.setBorder(BorderFactory.createRaisedBevelBorder());
		stop.setBorder(BorderFactory.createRaisedBevelBorder());
		JPanel buttons = new JPanel();
		buttons.setLayout(new GridLayout(1, 2));
		buttons.add(start);
		buttons.add(stop);

		port.setForeground(LABEL_COLOR);
		port_value.setForeground(LABEL_COLOR);
		status.setForeground(LABEL_COLOR);
		status_value.setForeground(OFFLINE_COLOR);
		
		Box infobox = Box.createHorizontalBox();
		infobox.add(status);
		infobox.add(Box.createHorizontalStrut(20));
		infobox.add(status_value);
		infobox.add(Box.createHorizontalStrut(80));
		infobox.add(port);
		infobox.add(Box.createHorizontalStrut(20));
		infobox.add(port_value);
		
		messages.setBackground(new Color(200, 200, 200));
		messages.setEditable(false);
		JScrollPane msgPane = new JScrollPane(messages, JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED,
			JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
		msgPane.setPreferredSize(new Dimension(500, 300));
		msgPane.setBorder(BorderFactory.createLoweredBevelBorder());		
		msgScrollBar = msgPane.getVerticalScrollBar();
		//msgScrollBar.addAdjustmentListener(this);
		messages.addComponentListener(this);
		
		errors.setBackground(new Color(200, 200, 200));
		errors.setEditable(false);
		JScrollPane errsPane = new JScrollPane(errors, JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED,
			JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
		errsPane.setPreferredSize(new Dimension(500, 100));
		errsPane.setBorder(BorderFactory.createLoweredBevelBorder());
		errsScrollBar = errsPane.getVerticalScrollBar();
		//errsScrollBar.addAdjustmentListener(this);
		errors.addComponentListener(this);

		Box msgPanel = Box.createVerticalBox();
		msgPanel.add(new JLabel("Messages"));
		msgPanel.add(msgPane);
		msgPanel.add(new JLabel("Errors"));
		msgPanel.add(errsPane);

		content.add(buttons, BorderLayout.NORTH);
		content.add(msgPanel, BorderLayout.CENTER);
		content.add(infobox, BorderLayout.SOUTH);
		
		addWindowListener(new WindowAdapter() {
			public void windowClosing(WindowEvent e) {
				System.exit(0);
			}
		} );

		start.addActionListener(this);
		stop.addActionListener(this);
	}
	
	public void componentResized(ComponentEvent e) {
		if (e.getSource() == messages) {
			msgScrollBar.setValue(msgScrollBar.getMaximum());
		} else if (e.getSource() == errors) {
			errsScrollBar.setValue(errsScrollBar.getMaximum());
		}
	}
	
	public void componentHidden(ComponentEvent e) {}
	public void componentShown(ComponentEvent e) {}
	public void componentMoved(ComponentEvent e) {}
	
	public JTextArea getOutputArea() {
		return messages;
	}
	
	public JTextArea getErrorOutputArea() {
		return errors;
	}
	
	public void actionPerformed(ActionEvent e) {
		if (e.getSource() == start) {
			if (server == null) {
				int port;
				String input = port_value.getText();
				if(input==null){
					Output.err.println("Port value = null using default "+server.DEFAULT_PORT);
					port=server.DEFAULT_PORT;
				}
				else{
					try{
						port=Integer.parseInt(input);
					}catch (NumberFormatException nf){
						Output.err.println("Port '"+port_value.getText()+"' is invalid using default "+server.DEFAULT_PORT);
						port=server.DEFAULT_PORT;
						port_value.setText(new Integer(port).toString());
					}
				}
				try {
					server = new HTTPServer(port);
					server.start();
					status_value.setForeground(ONLINE_COLOR);
					status_value.setText("Online");
					port_value.setEditable(false);
				} catch (Exception exc) {
					Output.err.println(exc.getMessage());
				}
			} else {
				Output.err.println("Server already online, disconnect first");
			}
		} else if (e.getSource() == stop) {
			if (server != null) {
				server.exit();
				server = null;
				status_value.setForeground(OFFLINE_COLOR);
				status_value.setText("Offline");
				port_value.setEditable(true);
			}
		}
	}
	
}